@@ -4,59 +4,38 @@ use 5.010;
 use strict;
 use warnings;
 
-our $VERSION = '0.52';
+use Carp qw< carp croak >;
+
+our $VERSION = '0.60';
 
 use Exporter;
-our @ISA            = qw( bigint );
+our @ISA            = qw( Exporter );
 our @EXPORT_OK      = qw( PI e bpi bexp hex oct );
 our @EXPORT         = qw( inf NaN );
 
 use overload;
-use bigint ();
+use Math::BigFloat;
 
 ##############################################################################
 
-BEGIN {
-    *inf = \&bigint::inf;
-    *NaN = \&bigint::NaN;
-    *hex = \&bigint::hex;
-    *oct = \&bigint::oct;
+sub accuracy {
+    my $self = shift;
+    Math::BigFloat -> accuracy(@_);
 }
 
-# These are all alike, and thus faked by AUTOLOAD
-
-my @faked = qw/round_mode accuracy precision div_scale/;
-our ($AUTOLOAD, $_lite);        # _lite for testsuite
-
-sub AUTOLOAD {
-    my $name = $AUTOLOAD;
-
-    $name =~ s/.*:://;          # split package
-    no strict 'refs';
-    foreach my $n (@faked) {
-        if ($n eq $name) {
-            *{"bignum::$name"} =
-              sub {
-                  my $self = shift;
-                  no strict 'refs';
-                  if (defined $_[0]) {
-                      Math::BigInt->$name($_[0]);
-                      return Math::BigFloat->$name($_[0]);
-                  }
-                  return Math::BigInt->$name();
-              };
-            return &$name;
-        }
-    }
-
-    # delayed load of Carp and avoid recursion
-    require Carp;
-    Carp::croak ("Can't call bignum\-\>$name, not a valid method");
+sub precision {
+    my $self = shift;
+    Math::BigFloat -> precision(@_);
 }
 
-sub unimport {
-    $^H{bignum} = undef;                                # no longer in effect
-    overload::remove_constant('binary', '', 'float', '', 'integer');
+sub round_mode {
+    my $self = shift;
+    Math::BigFloat -> round_mode(@_);
+}
+
+sub div_scale {
+    my $self = shift;
+    Math::BigFloat -> div_scale(@_);
 }
 
 sub in_effect {
@@ -65,140 +44,303 @@ sub in_effect {
     $hinthash->{bignum};
 }
 
+sub _float_constant {
+    my $str = shift;
+
+    # See if we can convert the input string to a string using a normalized form
+    # consisting of the significand as a signed integer, the character "e", and
+    # the exponent as a signed integer, e.g., "+0e+0", "+314e-2", and "-1e+3".
+
+    my $nstr;
+
+    if (
+        # See if it is an octal number. An octal number like '0377' is also
+        # accepted by the functions parsing decimal and hexadecimal numbers, so
+        # handle octal numbers before decimal and hexadecimal numbers.
+
+        $str =~ /^0(?:[Oo]|_*[0-7])/ and
+        $nstr = Math::BigInt -> oct_str_to_dec_flt_str($str)
+
+          or
+
+        # See if it is decimal number.
+
+        $nstr = Math::BigInt -> dec_str_to_dec_flt_str($str)
+
+          or
+
+        # See if it is a hexadecimal number. Every hexadecimal number has a
+        # prefix, but the functions parsing numbers don't require it, so check
+        # to see if it actually is a hexadecimal number.
+
+        $str =~ /^0[Xx]/ and
+        $nstr = Math::BigInt -> hex_str_to_dec_flt_str($str)
+
+          or
+
+        # See if it is a binary numbers. Every binary number has a prefix, but
+        # the functions parsing numbers don't require it, so check to see if it
+        # actually is a binary number.
+
+        $str =~ /^0[Bb]/ and
+        $nstr = Math::BigInt -> bin_str_to_dec_flt_str($str))
+    {
+        return Math::BigFloat -> new($nstr);
+    }
+
+    # If we get here, there is a bug in the code above this point.
+
+    warn "Internal error: unable to handle literal constant '$str'.",
+      " This is a bug, so please report this to the module author.";
+    return Math::BigFloat -> bnan();
+}
+
 #############################################################################
+# the following two routines are for "use bignum qw/hex oct/;":
+
+use constant LEXICAL => $] > 5.009004;
+
+# Internal function with the same semantics as CORE::hex(). This function is
+# not used directly, but rather by other front-end functions.
+
+sub _hex_core {
+    my $str = shift;
+
+    # Strip off, clean, and parse as much as we can from the beginning.
+
+    my $x;
+    if ($str =~ s/ ^ ( 0? [xX] )? ( [0-9a-fA-F]* ( _ [0-9a-fA-F]+ )* ) //x) {
+        my $chrs = $2;
+        $chrs =~ tr/_//d;
+        $chrs = '0' unless CORE::length $chrs;
+        $x = Math::BigFloat -> from_hex($chrs);
+    } else {
+        $x = Math::BigFloat -> bzero();
+    }
+
+    # Warn about trailing garbage.
+
+    if (CORE::length($str)) {
+        require Carp;
+        Carp::carp(sprintf("Illegal hexadecimal digit '%s' ignored",
+                           substr($str, 0, 1)));
+    }
+
+    return $x;
+}
+
+# Internal function with the same semantics as CORE::oct(). This function is
+# not used directly, but rather by other front-end functions.
+
+sub _oct_core {
+    my $str = shift;
+
+    $str =~ s/^\s*//;
+
+    # Hexadecimal input.
+
+    return _hex_core($str) if $str =~ /^0?[xX]/;
+
+    my $x;
+
+    # Binary input.
+
+    if ($str =~ /^0?[bB]/) {
+
+        # Strip off, clean, and parse as much as we can from the beginning.
+
+        if ($str =~ s/ ^ ( 0? [bB] )? ( [01]* ( _ [01]+ )* ) //x) {
+            my $chrs = $2;
+            $chrs =~ tr/_//d;
+            $chrs = '0' unless CORE::length $chrs;
+            $x = Math::BigFloat -> from_bin($chrs);
+        }
+
+        # Warn about trailing garbage.
+
+        if (CORE::length($str)) {
+            require Carp;
+            Carp::carp(sprintf("Illegal binary digit '%s' ignored",
+                               substr($str, 0, 1)));
+        }
+
+        return $x;
+    }
+
+    # Octal input. Strip off, clean, and parse as much as we can from the
+    # beginning.
+
+    if ($str =~ s/ ^ ( 0? [oO] )? ( [0-7]* ( _ [0-7]+ )* ) //x) {
+        my $chrs = $2;
+        $chrs =~ tr/_//d;
+        $chrs = '0' unless CORE::length $chrs;
+        $x = Math::BigFloat -> from_oct($chrs);
+    }
+
+    # Warn about trailing garbage. CORE::oct() only warns about 8 and 9, but it
+    # is more helpful to warn about all invalid digits.
+
+    if (CORE::length($str)) {
+        require Carp;
+        Carp::carp(sprintf("Illegal octal digit '%s' ignored",
+                           substr($str, 0, 1)));
+    }
+
+    return $x;
+}
+
+{
+    my $proto = LEXICAL ? '_' : ';$';
+    eval '
+sub hex(' . $proto . ') {' . <<'.';
+    my $str = @_ ? $_[0] : $_;
+    _hex_core($str);
+}
+.
+
+    eval '
+sub oct(' . $proto . ') {' . <<'.';
+    my $str = @_ ? $_[0] : $_;
+    _oct_core($str);
+}
+.
+}
+
+#############################################################################
+# the following two routines are for Perl 5.9.4 or later and are lexical
+
+my ($prev_oct, $prev_hex, $overridden);
+
+if (LEXICAL) { eval <<'.' }
+sub _hex(_) {
+    my $hh = (caller 0)[10];
+    return $$hh{bignum} ? bignum::_hex_core($_[0])
+         : $$hh{bigrat} ? bigrat::_hex_core($_[0])
+         : $$hh{bigint} ? bigint::_hex_core($_[0])
+         : $prev_hex    ? &$prev_hex($_[0])
+         : CORE::hex($_[0]);
+}
+
+sub _oct(_) {
+    my $hh = (caller 0)[10];
+    return $$hh{bignum} ? bignum::_oct_core($_[0])
+         : $$hh{bigrat} ? bigrat::_oct_core($_[0])
+         : $$hh{bigint} ? bigint::_oct_core($_[0])
+         : $prev_oct    ? &$prev_oct($_[0])
+         : CORE::oct($_[0]);
+}
+.
+
+sub _override {
+    return if $overridden;
+    $prev_oct = *CORE::GLOBAL::oct{CODE};
+    $prev_hex = *CORE::GLOBAL::hex{CODE};
+    no warnings 'redefine';
+    *CORE::GLOBAL::oct = \&_oct;
+    *CORE::GLOBAL::hex = \&_hex;
+    $overridden = 1;
+}
+
+sub unimport {
+    $^H{bignum} = undef;        # no longer in effect
+    overload::remove_constant('binary', '', 'float', '', 'integer');
+}
 
 sub import {
     my $self = shift;
 
-    $^H{bignum} = 1;                                    # we are in effect
+    $^H{bignum} = 1;                            # we are in effect
+    $^H{bigint} = undef;
+    $^H{bigrat} = undef;
 
-    # for newer Perls override hex() and oct() with a lexical version:
-    if ($] > 5.009004) {
-        bigint::_override();
+    # for newer Perls always override hex() and oct() with a lexical version:
+    if (LEXICAL) {
+        _override();
     }
 
     # some defaults
-    my $lib       = '';
-    my $lib_kind  = 'try';
-    my $upgrade   = 'Math::BigFloat';
-    my $downgrade = 'Math::BigInt';
+    my $lib      = '';
+    my $lib_kind = 'try';
 
-    my @import = (':constant');                 # drive it w/ constant
-    my @a = @_;
-    my $l = scalar @_;
-    my $j = 0;
+    my @import = ();
+    my @a = ();
     my ($ver, $trace);                          # version? trace?
-    my ($a, $p);                                # accuracy, precision
-    for (my $i = 0; $i < $l; $i++, $j++) {
-        if ($_[$i] eq 'upgrade') {
-            # this causes upgrading
-            $upgrade = $_[$i + 1];              # or undef to disable
-            my $s = 2;
-            $s = 1 if @a - $j < 2;  # avoid "can not modify non-existent..."
-            splice @a, $j, $s;
-            $j -= $s;
-            $i++;
-        } elsif ($_[$i] eq 'downgrade') {
-            # this causes downgrading
-            $downgrade = $_[$i + 1];            # or undef to disable
-            my $s = 2;
-            $s = 1 if @a - $j < 2;  # avoid "can not modify non-existent..."
-            splice @a, $j, $s;
-            $j -= $s;
-            $i++;
-        } elsif ($_[$i] =~ /^(l|lib|try|only)$/) {
-            # this causes a different low lib to take care...
-            $lib_kind = $1;
-            $lib_kind = 'lib' if $lib_kind eq 'l';
-            $lib = $_[$i + 1] || '';
-            my $s = 2;
-            $s = 1 if @a - $j < 2;  # avoid "can not modify non-existent..."
-            splice @a, $j, $s;
-            $j -= $s;
-            $i++;
-        }
-        elsif ($_[$i] =~ /^(a|accuracy)$/) {
-            $a = $_[$i + 1];
-            my $s = 2;
-            $s = 1 if @a - $j < 2;  # avoid "can not modify non-existent..."
-            splice @a, $j, $s;
-            $j -= $s;
-            $i++;
-        }
-        elsif ($_[$i] =~ /^(p|precision)$/) {
-            $p = $_[$i + 1];
-            my $s = 2;
-            $s = 1 if @a - $j < 2;  # avoid "can not modify non-existent..."
-            splice @a, $j, $s;
-            $j -= $s;
-            $i++;
-        }
-        elsif ($_[$i] =~ /^(v|version)$/) {
-            $ver = 1;
-            splice @a, $j, 1;
-            $j--;
-        }
-        elsif ($_[$i] =~ /^(t|trace)$/) {
-            $trace = 1;
-            splice @a, $j, 1;
-            $j--;
-        }
-        elsif ($_[$i] !~ /^(PI|e|bexp|bpi|hex|oct)\z/) {
-            die ("unknown option $_[$i]");
-        }
-    }
-    my $class;
-    $_lite = 0;                                 # using M::BI::L ?
-    if ($trace) {
-        require Math::BigInt::Trace;
-        $class   = 'Math::BigInt::Trace';
-        $upgrade = 'Math::BigFloat::Trace';
-    }
-    else {
-        # see if we can find Math::BigInt::Lite
-        if (!defined $a && !defined $p) {       # rounding won't work to well
-            local @INC = @INC;
-            pop @INC if $INC[-1] eq '.';
-            if (eval { require Math::BigInt::Lite; 1 }) {
-                @import = ();                   # :constant in Lite, not MBI
-                Math::BigInt::Lite->import(':constant');
-                $_lite = 1;                     # signal okay
-            }
-        }
-        require Math::BigInt if $_lite == 0;    # not already loaded?
-        $class = 'Math::BigInt';                # regardless of MBIL or not
-    }
-    push @import, $lib_kind => $lib if $lib ne '';
-    # Math::BigInt::Trace or plain Math::BigInt
-    $class->import(@import, upgrade => $upgrade);
 
+    while (@_) {
+        my $param = shift;
+        if ($param eq 'upgrade') {
+            # this causes upgrading
+            push @import, 'upgrade', shift;
+        } elsif ($param eq 'downgrade') {
+            # this causes downgrading
+            push @import, 'downgrade', shift;
+        } elsif ($param =~ /^(l|lib|try|only)$/) {
+            # this causes a different low lib to take care...
+            $lib_kind = $param;
+            $lib_kind = 'lib' if $lib_kind eq 'l';
+            $lib = shift() || '';
+        } elsif ($param =~ /^(a|accuracy)$/) {
+            push @import, 'accuracy', shift;
+        } elsif ($param =~ /^(p|precision)$/) {
+            push @import, 'precision', shift;
+        } elsif ($param =~ /^(v|version)$/) {
+            $ver = 1;
+        } elsif ($param =~ /^(t|trace)$/) {
+            $trace = 1;
+        } elsif ($param =~ /^(PI|e|bexp|bpi|hex|oct)\z/) {
+            push @a, $param;
+        } else {
+            croak("Unknown option '$param'");
+        }
+    }
+
+    my $class = "Math::BigFloat";
     if ($trace) {
         require Math::BigFloat::Trace;
-        $class     = 'Math::BigFloat::Trace';
-        $downgrade = 'Math::BigInt::Trace';
+        $class = "Math::BigFloat::Trace";
     }
-    else {
-        require Math::BigFloat;
-        $class = 'Math::BigFloat';
-    }
-    $class->import(':constant', 'downgrade', $downgrade);
+    push @import, $lib_kind => $lib if $lib ne '';
+    $class -> import(@import);
 
-    bignum->accuracy($a)  if defined $a;
-    bignum->precision($p) if defined $p;
     if ($ver) {
         print "bignum\t\t\t v$VERSION\n";
-        print "Math::BigInt::Lite\t v$Math::BigInt::Lite::VERSION\n" if $_lite;
-        print "Math::BigInt\t\t v$Math::BigInt::VERSION";
-        my $config = Math::BigInt->config();
+        my $config = $class -> config();
         print " lib => $config->{lib} v$config->{lib_version}\n";
-        print "Math::BigFloat\t\t v$Math::BigFloat::VERSION\n";
+        print $class, "\t\t v", $class -> VERSION, "\n";
         exit;
     }
 
-    # Take care of octal/hexadecimal constants
-    overload::constant binary =>
-        sub {
-            bigint::_binary_constant(shift);
+    overload::constant
+
+        # This takes care each number written as decimal integer and within the
+        # range of what perl can represent as an integer, e.g., "314", but not
+        # "3141592653589793238462643383279502884197169399375105820974944592307".
+
+        integer => sub {
+            #printf "Value '%s' handled by the 'integer' sub.\n", $_[0];
+            my $str = shift;
+            return Math::BigFloat -> new($str);
+        },
+
+        # This takes care of each number written with a decimal point and/or
+        # using floating point notation, e.g., "3.", "3.0", "3.14e+2" (decimal),
+        # "0b1.101p+2" (binary), "03.14p+2" and "0o3.14p+2" (octal), and
+        # "0x3.14p+2" (hexadecimal).
+
+        float => sub {
+            #printf "# Value '%s' handled by the 'float' sub.\n", $_[0];
+            _float_constant(shift);
+        },
+
+        # Take care of each number written as an integer (no decimal point or
+        # exponent) using binary, octal, or hexadecimal notation, e.g., "0b101"
+        # (binary), "0314" and "0o314" (octal), and "0x314" (hexadecimal).
+
+        binary => sub {
+            #printf "# Value '%s' handled by the 'binary' sub.\n", $_[0];
+            my $str = shift;
+            return Math::BigFloat -> new($str) if $str =~ /^0[XxBb]/;
+            Math::BigFloat -> from_oct($str);
         };
 
     # if another big* was already loaded:
@@ -210,12 +352,17 @@ sub import {
     }
 }
 
-sub PI () { Math::BigFloat->new('3.141592653589793238462643383279502884197'); }
-sub e () { Math::BigFloat->new('2.718281828459045235360287471352662497757'); }
-sub bpi ($) { Math::BigFloat->bpi(@_); }
+sub inf () { Math::BigFloat -> binf(); }
+sub NaN () { Math::BigFloat -> bnan(); }
+
+# This should depend on the current accuracy/precision. Fixme!
+sub PI  () { Math::BigFloat -> new('3.141592653589793238462643383279502884197'); }
+sub e   () { Math::BigFloat -> new('2.718281828459045235360287471352662497757'); }
+
+sub bpi ($) { Math::BigFloat -> bpi(@_); }
 sub bexp ($$) {
-    my $x = Math::BigFloat->new($_[0]);
-    $x->bexp($_[1]);
+    my $x = Math::BigFloat -> new($_[0]);
+    $x -> bexp($_[1]);
 }
 
 1;
@@ -226,13 +373,13 @@ __END__
 
 =head1 NAME
 
-bignum - Transparent BigNumber support for Perl
+bignum - transparent big number support for Perl
 
 =head1 SYNOPSIS
 
     use bignum;
 
-    $x = 2 + 4.5,"\n";              # BigFloat 6.5
+    $x = 2 + 4.5,"\n";              # Math::BigFloat 6.5
     print 2 ** 512 * 0.1,"\n";      # really is what you think it is
     print inf * inf,"\n";           # prints inf
     print NaN * 3,"\n";             # prints NaN
@@ -249,46 +396,24 @@ bignum - Transparent BigNumber support for Perl
 
 =head1 DESCRIPTION
 
-All operators (including basic math operations) are overloaded. Integer and
-floating-point constants are created as proper BigInts or BigFloats,
-respectively.
+All operators (including basic math operations) except the range operator C<..>
+are overloaded. All literal numeric constants are converted to Math::BigFloat
+objects.
 
-If you do
+So, the following:
 
     use bignum;
-
-at the top of your script, Math::BigFloat and Math::BigInt will be loaded
-and any constant number will be converted to an object (Math::BigFloat for
-floats like 3.1415 and Math::BigInt for integers like 1234).
-
-So, the following line:
-
     $x = 1234;
 
-creates actually a Math::BigInt and stores a reference to in $x.
-This happens transparently and behind your back, so to speak.
+creates a Math::BigFloat and stores a reference to in $x. This happens
+transparently and behind your back, so to speak.
 
 You can see this with the following:
 
     perl -Mbignum -le 'print ref(1234)'
 
-Don't worry if it says Math::BigInt::Lite, bignum and friends will use Lite
-if it is installed since it is faster for some operations. It will be
-automatically upgraded to BigInt whenever necessary:
-
-    perl -Mbignum -le 'print ref(2**255)'
-
-This also means it is a bad idea to check for some specific package, since
-the actual contents of $x might be something unexpected. Due to the
-transparent way of bignum C<ref()> should not be necessary, anyway.
-
-Since Math::BigInt and BigFloat also overload the normal math operations,
-the following line will still work:
-
-    perl -Mbignum -le 'print ref(1234+1234)'
-
 Since numbers are actually objects, you can call all the usual methods from
-BigInt/BigFloat on them. This even works to some extent on expressions:
+Math::BigFloat on them. This even works to some extent on expressions:
 
     perl -Mbignum -le '$x = 1234; print $x->bdec()'
     perl -Mbignum -le 'print 1234->copy()->binc();'
@@ -303,44 +428,6 @@ You can even chain the operations together as usual:
     perl -Mbignum -le 'print 1234->copy()->binc->badd(6);'
     1241
 
-Under bignum (or bigint or bigrat), Perl will "upgrade" the numbers
-appropriately. This means that:
-
-    perl -Mbignum -le 'print 1234+4.5'
-    1238.5
-
-will work correctly. These mixed cases don't do always work when using
-Math::BigInt or Math::BigFloat alone, or at least not in the way normal Perl
-scalars work.
-
-If you do want to work with large integers like under C<use integer;>, try
-C<use bigint;>:
-
-    perl -Mbigint -le 'print 1234.5+4.5'
-    1238
-
-There is also C<use bigrat;> which gives you big rationals:
-
-    perl -Mbigrat -le 'print 1234+4.1'
-    12381/10
-
-The entire upgrading/downgrading is still experimental and might not work
-as you expect or may even have bugs. You might get errors like this:
-
-    Can't use an undefined value as an ARRAY reference at
-    /usr/local/lib/perl5/5.8.0/Math/BigInt/Calc.pm line 864
-
-This means somewhere a routine got a BigFloat/Lite but expected a BigInt (or
-vice versa) and the upgrade/downgrad path was missing. This is a bug, please
-report it so that we can fix it.
-
-You might consider using just Math::BigInt or Math::BigFloat, since they
-allow you finer control over what get's done in which module/space. For
-instance, simple loop counters will be Math::BigInts under C<use bignum;> and
-this is slower than keeping them as Perl scalars:
-
-    perl -Mbignum -le 'for ($i = 0; $i < 10; $i++) { print ref($i); }'
-
 Please note the following does not work as expected (prints nothing), since
 overloading of '..' is not yet possible in Perl (as of v5.8.0):
 
@@ -419,9 +506,9 @@ This prints out the name and version of all modules used and then exits.
 Beside import() and AUTOLOAD() there are only a few other methods.
 
 Since all numbers are now objects, you can use all functions that are part of
-the BigInt or BigFloat API. It is wise to use only the bxxx() notation, and not
-the fxxx() notation, though. This makes it possible that the underlying object
-might morph into a different class than BigFloat.
+the Math::BigInt or Math::BigFloat API. It is wise to use only the bxxx()
+notation, and not the fxxx() notation, though. This makes it possible that the
+underlying object might morph into a different class than Math::BigFloat.
 
 =head2 Caveats
 
@@ -459,18 +546,18 @@ Using methods that do not modify, but test the contents works:
     $z = 9 if $x->is_zero();                # works fine
 
 See the documentation about the copy constructor and C<=> in overload, as
-well as the documentation in BigInt for further details.
+well as the documentation in Math::BigInt for further details.
 
 =over 2
 
 =item inf()
 
-A shortcut to return Math::BigInt->binf(). Useful because Perl does not always
+A shortcut to return Math::BigFloat->binf(). Useful because Perl does not always
 handle bareword C<inf> properly.
 
 =item NaN()
 
-A shortcut to return Math::BigInt->bnan(). Useful because Perl does not always
+A shortcut to return Math::BigFloat->bnan(). Useful because Perl does not always
 handle bareword C<NaN> properly.
 
 =item e
@@ -508,8 +595,7 @@ Example:
 
 =item upgrade()
 
-Return the class that numbers are upgraded to, is in fact returning
-C<$Math::BigInt::upgrade>.
+Return the class that numbers are upgraded to, if any.
 
 =item in_effect()
 
@@ -597,10 +683,9 @@ For example:
     my $y = "900000000000000007";
     print $x - $y;
 
-will output C<0> on default 32-bit builds, since C<bigrat> never sees
-the string literals.  To ensure the expression is all treated as
-C<Math::BigInt> or C<BigFloat> objects, use a literal number in the
-expression:
+will output C<0> on default 32-bit builds, since C<bigrat> never sees the string
+literals. To ensure the expression is all treated as C<Math::BigInt> or
+C<Math::BigFloat> objects, use a literal number in the expression:
 
     print +(0+$x) - $y;
 
@@ -635,18 +720,6 @@ Compare this to:
 
 =back
 
-=head1 MODULES USED
-
-C<bignum> is just a thin wrapper around various modules of the Math::BigInt
-family. Think of it as the head of the family, who runs the shop, and orders
-the others to do the work.
-
-The following modules are currently used by bignum:
-
-    Math::BigInt::Lite      (for speed, and only if it is loadable)
-    Math::BigInt
-    Math::BigFloat
-
 =head1 EXAMPLES
 
 Some cool command line examples to impress the Python crowd ;)
